"""
.. module:: varnames
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Module that contains an enumeration of environment variable names
               that are used by the configuration code.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []

import os

from mojo.startup.presencevariables import (
    MOJO_PRESENCE_VARIABLES,
    resolve_presence_variables
)

from mojo.collections.wellknown import ContextSingleton
from mojo.collections.contextpaths import ContextPaths
from mojo.collections.mergemap import MergeMap


from mojo.config.normalize import (
    split_and_normalize_name_list,
    split_and_normalize_path_list,
    split_and_normalize_source_list
)

from mojo.config.configurationsettings import MOJO_CONFIG_DEFAULTS


class CONFIGURATION_MAPS:
    CREDENTIAL_CONFIGURATION_MAP = MergeMap()
    LANDSCAPE_CONFIGURATION_MAP = MergeMap()
    RUNTIME_CONFIGURATION_MAP = MergeMap(MOJO_CONFIG_DEFAULTS.DEFAULT_CONFIGURATION)
    TOPOLOGY_CONFIGURATION_MAP = MergeMap()


class MOJO_CONFIG_VARNAMES:

    MJR_NAME = "MJR_NAME"

    MJR_HOME_DIRECTORY = "MJR_HOME_DIRECTORY"

    MJR_CONFIG_DIRECTORY = "MJR_CONFIG_DIRECTORY"

    MJR_CONFIG_PASS_PHRASE = "MJR_CONFIG_PASS_PHRASE"

    MJR_CONFIG_STORAGE_URI = "MJR_CONFIG_STORAGE_URI"

    MJR_CONFIG_CREDENTIAL_NAMES = "MJR_CONFIG_CREDENTIAL_NAMES"
    MJR_CONFIG_CREDENTIAL_URIS = "MJR_CONFIG_CREDENTIAL_URIS"
    MJR_CONFIG_CREDENTIAL_SOURCES = "MJR_CONFIG_CREDENTIAL_SOURCES"
    MJR_CONFIG_CREDENTIAL_FILES = "MJR_CONFIG_CREDENTIAL_FILES"

    MJR_CONFIG_LANDSCAPE_NAMES = "MJR_CONFIG_LANDSCAPE_NAMES"
    MJR_CONFIG_LANDSCAPE_URIS = "MJR_CONFIG_LANDSCAPE_URIS"
    MJR_CONFIG_LANDSCAPE_SOURCES = "MJR_CONFIG_LANDSCAPE_SOURCES"
    MJR_CONFIG_LANDSCAPE_FILES = "MJR_CONFIG_LANDSCAPE_FILES"

    MJR_CONFIG_TOPOLOGY_NAMES = "MJR_CONFIG_TOPOLOGY_NAMES"
    MJR_CONFIG_TOPOLOGY_URIS = "MJR_CONFIG_TOPOLOGY_URIS"
    MJR_CONFIG_TOPOLOGY_SOURCES = "MJR_CONFIG_TOPOLOGY_SOURCES"
    MJR_CONFIG_TOPOLOGY_FILES = "MJR_CONFIG_TOPOLOGY_FILES"

    MJR_CONFIG_RUNTIME_NAMES = "MJR_CONFIG_RUNTIME_NAMES"
    MJR_CONFIG_RUNTIME_URIS = "MJR_CONFIG_RUNTIME_URIS"
    MJR_CONFIG_RUNTIME_SOURCES = "MJR_CONFIG_RUNTIME_SOURCES"
    MJR_CONFIG_RUNTIME_FILES = "MJR_CONFIG_RUNTIME_FILES"


class MOJO_CONFIG_VARIABLES(MOJO_PRESENCE_VARIABLES):

    MJR_CONFIG_REQUIRE_CREDENTIALS = MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_CREDENTIALS
    MJR_CONFIG_REQUIRE_LANDSCAPE = MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_LANDSCAPE
    MJR_CONFIG_REQUIRE_RUNTIME = MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_RUNTIME
    MJR_CONFIG_REQUIRE_TOPOLOGY = MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_TOPOLOGY

    MJR_CONFIG_STORAGE_URI = MOJO_CONFIG_DEFAULTS.MJR_CONFIG_STORAGE_URI

    MJR_CONFIG_USE_CREDENTIALS = False
    MJR_CONFIG_USE_LANDSCAPE = False
    MJR_CONFIG_USE_RUNTIME = False
    MJR_CONFIG_USE_TOPOLOGY = False

    MJR_CONFIG_DIRECTORY = os.path.join(MOJO_PRESENCE_VARIABLES.MJR_HOME_DIRECTORY, "config")

    MJR_CONFIG_PASS_PHRASE = None

    MJR_CONFIG_CREDENTIAL_NAMES = None
    MJR_CONFIG_CREDENTIAL_URIS = None
    MJR_CONFIG_CREDENTIAL_SOURCES = None
    MJR_CONFIG_CREDENTIAL_FILES = None

    MJR_CONFIG_LANDSCAPE_NAMES = None
    MJR_CONFIG_LANDSCAPE_URIS = None
    MJR_CONFIG_LANDSCAPE_SOURCES = None
    MJR_CONFIG_LANDSCAPE_FILES = None

    MJR_CONFIG_TOPOLOGY_NAMES = None
    MJR_CONFIG_TOPOLOGY_URIS = None
    MJR_CONFIG_TOPOLOGY_SOURCES = None
    MJR_CONFIG_TOPOLOGY_FILES = None

    MJR_CONFIG_RUNTIME_NAMES = None
    MJR_CONFIG_RUNTIME_URIS = None
    MJR_CONFIG_RUNTIME_SOURCES = None
    MJR_CONFIG_RUNTIME_FILES = None


def resolve_configuration_variables():

    # We need to resolve extension variables because we are
    # dependent on some of them.
    resolve_presence_variables()

    environ = os.environ

    ctx = ContextSingleton()

    # For now, we don't want to put the CREDENTIALS CONFIGURATION into the context, because at the moment we don't 
    # want to pass it around in the plain.  Possibly what we can do is create a version of MergeMap that will serialize
    # with a protective pass key or something like that.
    ctx.insert(ContextPaths.CONFIG_LANDSCAPE, CONFIGURATION_MAPS.LANDSCAPE_CONFIGURATION_MAP)
    ctx.insert(ContextPaths.CONFIG_TOPOLOGY, CONFIGURATION_MAPS.TOPOLOGY_CONFIGURATION_MAP)
    ctx.insert(ContextPaths.CONFIG_RUNTIME, CONFIGURATION_MAPS.RUNTIME_CONFIGURATION_MAP)


    MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY = os.path.join(MOJO_CONFIG_VARIABLES.MJR_HOME_DIRECTORY, "config")
    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_DIRECTORY in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY = environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_DIRECTORY]
    ctx.insert(ContextPaths.RUNTIME_CONFIG_DIRECTORY, MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY)


    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_STORAGE_URI in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_STORAGE_URI = environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_STORAGE_URI]
    ctx.insert(ContextPaths.CONFIG_STORAGE_URI, MOJO_CONFIG_VARIABLES.MJR_CONFIG_STORAGE_URI)

    # ====================== PROCESS CREDENTIAL VARIABLES ==============================

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_FILES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_CREDENTIALS = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_FILES = split_and_normalize_path_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_FILES])

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_NAMES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_CREDENTIALS = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_NAMES = split_and_normalize_name_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_NAMES])
    elif MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_CREDENTIALS and not MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_FILES:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_CREDENTIALS = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_NAMES = ["credentials"]

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_SOURCES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_SOURCES = split_and_normalize_source_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_CREDENTIAL_SOURCES])
    else:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_CREDENTIAL_SOURCES = [os.path.join(MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY)]

    # ====================== PROCESS LANDSCAPE VARIABLES ==============================

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_FILES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_LANDSCAPE = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_FILES = split_and_normalize_path_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_FILES])

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_NAMES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_LANDSCAPE = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_NAMES = split_and_normalize_name_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_NAMES])
    elif MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_LANDSCAPE and not MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_FILES:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_LANDSCAPE = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_NAMES = ["default-landscape"]

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_SOURCES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_SOURCES = split_and_normalize_source_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_LANDSCAPE_SOURCES])
    else:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_LANDSCAPE_SOURCES = [os.path.join(
            MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY, "landscapes")]

    # ====================== PROCESS RUNTIME VARIABLES ==============================

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_FILES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_RUNTIME = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_FILES = split_and_normalize_path_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_FILES])

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_NAMES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_RUNTIME = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_NAMES = split_and_normalize_name_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_NAMES])
    elif MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_RUNTIME and not MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_FILES:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_RUNTIME = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_NAMES = ["default-runtime"]

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_SOURCES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_SOURCES = split_and_normalize_source_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_RUNTIME_SOURCES])
    else:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_SOURCES = [os.path.join(
            MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY, "runtimes")]

    # ====================== PROCESS TOPOLOGY VARIABLES ==============================

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_FILES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_TOPOLOGY = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_TOPOLOGY_FILES = split_and_normalize_path_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_FILES])

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_NAMES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_TOPOLOGY = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_TOPOLOGY_NAMES = split_and_normalize_name_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_NAMES])
    elif MOJO_CONFIG_DEFAULTS.MJR_CONFIG_REQUIRE_RUNTIME and not MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_FILES:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_USE_RUNTIME = True
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_RUNTIME_NAMES = ["default-topology"]

    if MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_SOURCES in environ:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_TOPOLOGY_SOURCES = split_and_normalize_source_list(
            environ[MOJO_CONFIG_VARNAMES.MJR_CONFIG_TOPOLOGY_SOURCES])
    else:
        MOJO_CONFIG_VARIABLES.MJR_CONFIG_TOPOLOGY_SOURCES = [os.path.join(
            MOJO_CONFIG_VARIABLES.MJR_CONFIG_DIRECTORY, "topologies")]

    return
